// Show list of stories (later stories and defects) that were blocked between an arbitrary time range.
//API Docs: https://help.rallydev.com/apps/2.1/doc/

//
// ────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────── I ──────────
//   :::::: O U T E R   P A N E L   F O R   E N T I R E   T I M E   F R A M E   S P E C I F I C A T I O N   S E C T I O N : :  :   :    :     :        :          :
// ────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
//


this.myApp = Ext.define('CustomApp', {
    extend: 'Rally.app.App',
    componentCls: 'app',

    releaseTimeBox: Ext.create('Rally.ui.combobox.ReleaseComboBoxView', {
        flex: 1,
        name: 'release',
        itemId: 'release',
        fieldLabel: 'Release: ',
        padding: 6,
        border: true,
        borderWidth: 4,
        listeners: {
            select: _releaseChanged,
            scope: this
        }
    }),

    iterationTimeBox: Ext.create('Rally.ui.combobox.ReleaseComboBox', {
        name: 'iteration',
        itemId: 'iteration',
        fieldLabel: 'Iteration: ',
        padding: 6,
        border: true,
        borderWidth: 4,
        listeners: {
            select: _iterationChanged,
            scope: this
        }
    }),

    timeBoxesPanel: Ext.create('Ext.panel.Panel', {
        flex: 1,
        name: 'timeboxesPanel',
        itemId: 'timeBoxesPanel',
        title: 'Timeboxes',
        titleAlign: 'center',
        border: true,
        borderWidth: 4,
        items: [
            this.releaseTimeBox,
            this.iterationTimeBox
        ]
    }),

    startDateField: Ext.create('Rally.ui.DateField', {
        name: 'startDate',
        itemId: 'startDate',
        fieldLabel: 'Start Date',
        padding: 6,
        border: true,
        borderWidth: 4,
        listeners: {
            change: _startDateChanged,
            scope: this
        }
    }),

    endDateField: Ext.create('Rally.ui.Datefield', {
        name: 'endDate',
        itemId: 'endDate',
        fieldLabel: 'End Date',
        padding: 6,
        border: true,
        borderWidth: 4,
        listeners: {
            change: _endDateChanged,
            scope: this
        }
    }),

    dateRangePanel: Ext.create('Ext.panel.Panel', {
        flex: 1,
        name: 'dateRangePanel',
        itemId: 'dateRangePanel',
        title: 'Date Range',
        titleAlign: 'center',
        items: [
            this.startDateField,
            this.endDateField
        ],
        scope: this
    }),

    topContainer: Ext.create('Ext.panel.Panel', {
        layout: {
            type: 'hbox',
            align: 'stretch'
        },
        name: 'timeFramePanel',
        itemId: 'timeFramePanel',
        width: '100%',
        title: 'Time Frame',
        titleAlign: 'center',
        renderTo: Ext.getBody(),
        border: true,
        style: {
            borderColor: '#000000',
            borderStyle: 'solid',
            borderWidth: '1px'
        },
        defaults: {
            labelWidth: 80,
            align: 'stretch',
            flex: 1,
            style: {
                padding: '12px'
            },
            scope: this,
            items: [
                this.timeBoxesPanel, 
                this.dateRangePanel
            ]
        }
    }),
    launch: function () {
        //Write app code here
        console.log("Entering launch function.");
        console.log("Attempt to add top container to app");
        this.add(this.topContainer);
    },
});

Ext.onReady(function () {
    console.log("Entering onReady");
    console.log("This is: ", this);
});

// this.myApp.add(this.topContainer);
// this.topContainer.add([
//     this.timeBoxesPanel,
//     this.dateRangePanel
// ]);
// this.timeBoxesPanel.add([
//     this.startDateField,
//     this.endDateField
// ]);


//
// ─── CHANGES TO THE TIMEBOX FIELDS ──────────────────────────────────────────────
//

function _releaseChanged(rallyreleasecombobox, release, eOpts) {
    // Get the current timebox scope. Will be undefined if the app is not running in a timebox filtered custom page.
    // Set the timebox scope to release and to the value specified.    

    console.log("Release changed: ", release);
    console.log("Resetting iteration to undefined");
}

function _iterationChanged(rallyiterationcombobox, iteration, eOpts) {
    console.log("Iteration changed: ", iteration);
    // set timebox scope to type iteration and to the value specified.

    // set the start and end dates to the start and end dates of the timebox.
}

//
// ─── CHANGES TO THE DATE RANGE FIELDS ─────────────────────────────────────────────────
//

function _startDateChanged(datefield, newDate, eOpts) {
    _setTimeBoxScope(undefined);
    // reset timeboxes to none
    console.log("Start Date changed: ", newDate);
}

function _endDateChanged(datefield, newDate, eOpts) {
    console.log("End Date changed: ", newDate);

    // validate that end date is >= start date
    var startDate = Ext.ComponentQuery.query('#startDate')[0];
    if (datefield.value < startDate.value) {
        console.log("End Date < Start Date");
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// If record is undefined or null, set TimeBoxScope to undefined; otherwise, set the TimeBox to record. //
//////////////////////////////////////////////////////////////////////////////////////////////////////////
function _setTimeBoxScope(record) {
    if ((record === undefined) || (record === null)) {
        // set TimeBoxScope to undefined and TimeBoxScopeType to  undefined
        console.log("Resetting TimeBox Scope");
    }
}

////////////////////////////////////////////////////
// Reset both start and end date fields to empty. //
////////////////////////////////////////////////////
function _clearDateRange() {
    console.log("Resetting Date Range Fields.");
}